
#pragma warning(disable:4996)
#include "Socket.h"
#include "SocketDlg.h"
#include <tchar.h>
#include <time.h>



// Constructor
CSocketDlg::CSocketDlg()
{
	b_Server = true;
	Terminated = false;
	InitializeCriticalSection(&mk_Critical);
	allpacketscount = 0;
}


CSocketDlg::~CSocketDlg()
{
	DeleteCriticalSection(&mk_Critical);
}

void CSocketDlg::SetPort(unsigned short Port)
{
	ms32_Port = Port;
}

// -------------------------------------------------------------------------

// Switch this application into Server mode and listen for Client connections
DWORD CSocketDlg::Start() 
{
	if (mi_Socket.GetSocketCount())
	{
		Print("Socket already in use!");
		return -1;
	}
	//mi_Socket.GetSocketCount
	allpacketscount = 0;
	DWORD u32_BindIP = 0;
	
	std::string s_BindIP;

	DWORD u32_EventTimeout = (PROCESS_EVENTS_IN_GUI_THREAD) ? 50 : INFINITE;

	DWORD u32_Err = mi_Socket.Listen(u32_BindIP, ms32_Port, u32_EventTimeout, MAX_SERVER_IDLE_TIME);
	
	if (u32_Err) {Print("Listen Error %s", GetErrMsg(u32_Err).c_str());return u32_Err ;}
	else
	{
		Print("Listening (%s) on Port %d. Max count of clients: %lu", s_BindIP.c_str(), ms32_Port,MAXCLIENTS);

		if (MAX_SERVER_IDLE_TIME > 0)
			Print("Maximum idle time per client: %d seconds", MAX_SERVER_IDLE_TIME);
	}
	
	if (u32_Err) 
	{
		CloseSockets();
		return u32_Err;
	}

	// runs until an error occurred or all sockets have closed
	#if PROCESS_EVENTS_IN_GUI_THREAD
		ProcessEvents();
	#else
		DWORD u32_ID;
		mh_Thread = CreateThread(0, 0, ProcessEventThread, this, 0, &u32_ID);
	#endif
		return u32_Err;
}

// Switch this application into Client mode and connect to a server
void CSocketDlg::OnBtnConnect() 
{
	if (mi_Socket.GetSocketCount())
	{
		Print("Socket already in use!");
		return;
	}

	DWORD u32_IP=0;
	//mi_IpAddr.GetAddress(u32_IP);
	u32_IP = htonl(u32_IP);

	DWORD u32_EventTimeout = (PROCESS_EVENTS_IN_GUI_THREAD) ? 50 : INFINITE;

	DWORD u32_Err = mi_Socket.ConnectTo(u32_IP, ms32_Port, u32_EventTimeout, MAX_CLIENT_IDLE_TIME);
	
	if (u32_Err) Print("Connect Error %s", GetErrMsg(u32_Err));
	else
	{
		Print("Connecting to Server (%s) on Port %d.... (waiting for FD_CONNECT)", FormatIP(u32_IP).c_str(), ms32_Port);

		if (MAX_CLIENT_IDLE_TIME > 0)
			Print("Maximum idle time: %d seconds", MAX_CLIENT_IDLE_TIME);
	}

	if (u32_Err) 
	{
		CloseSockets();
		return;
	}

	// runs until an error occurred or all sockets have closed
	#if PROCESS_EVENTS_IN_GUI_THREAD
		ProcessEvents();
	#else
		DWORD u32_ID;
		mh_Thread = CreateThread(0, 0, ProcessEventThread, this, 0, &u32_ID);
	#endif
}

// static
ULONG WINAPI CSocketDlg::ProcessEventThread(void* p_Param)
{
	
	CSocketDlg* p_This = (CSocketDlg*)p_Param;
	while (true)
	{
		p_This->ProcessEvents(p_This->Terminated);
		
	}
	CloseHandle(p_This->mh_Thread);
	
	return 0;
}

// Process all events which occur on one of the open sockets
void CSocketDlg::ProcessEvents(bool Terminated)
{
	try
	{
	while (TRUE) // Loop runs until the main window was closed or a severe error occurred
	{
		#if PROCESS_EVENTS_IN_GUI_THREAD
			PumpMessages();
		#endif
		


		TCP::cSocket::cMemory* pi_RecvMem;
		SOCKET  h_Socket;
		DWORD tmpsize = 0;
		unsigned char* tmpdata = NULL;
		DWORD u32_Event, u32_IP, u32_Read, u32_Sent;
		DWORD u32_Err = mi_Socket.ProcessEvents(&u32_Event, &u32_IP, 
			&h_Socket, &pi_RecvMem, &u32_Read,  &u32_Sent,&tmpsize,&tmpdata);
		if (Terminated) 
			return;  // return NOT break!

		if (u32_Err == ERROR_TIMEOUT) // 50 ms interval has elapsed
			continue;
		
		std::string s_Msg, s_Events;
		if (u32_Event) // ATTENTION: u32_Event may be == 0 -> do nothing.
		{

			char s8_Events[1024];
			mi_Socket.FormatEvents(u32_Event, s8_Events);
			
			s_Events += s8_Events;
			Print(s_Events.c_str());

			if (u32_Event & FD_READ)  
				{
					char tmp[128] = "\0";
					memset(&tmp,0,sizeof(tmp));
					//sprintf(tmp,"%d Bytes received.\n",u32_Read);
					//s_Msg = tmp;
					{
						switch (me_DemoMode)
						{
							//case E_NORMAL: ProcessReceivedDataNormal(*&pi_RecvMem); break;
							case E_NORMAL: 
							{
								ProcessReceivedData(tmpdata,tmpsize,h_Socket,*&pi_RecvMem); 								
								break;
							}
							case E_PREFIXED: ProcessReceivedDataPrefix(pi_RecvMem); break;
							case E_TELNET:   ProcessReceivedDataTelnet(pi_RecvMem); break;
							default:Print("Not Parsed current data"); break;
						}
						if (tmpdata) {free(tmpdata); tmpdata = NULL;}
					}
				}
			//Print("%s",s_Msg.c_str());
		}
		
		// It is NOT necessary to update the Combobox after FD_READ or FD_WRITE
		mb_RefreshCombo |= (u32_Event & (FD_ACCEPT | FD_CONNECT | FD_CLOSE) || u32_Err);

		if (u32_Err)
		{
			// mi_Socket.Close() has been called -> don't print this error message
			if (u32_Err == WSAENOTCONN)
				break;

			// Print all the other error messages
			//Print("ProcessEvent Error %s\n", GetErrMsg(u32_Err).c_str());
			
			// An error normally means that the socket has a problem -> abort the loop.
			// A few errors should not abort the processing:
			if (u32_Err != WSAECONNABORTED && // e.g. after the other side was killed in TaskManager 
				u32_Err != WSAECONNRESET   && // Connection reset by peer.
				u32_Err != WSAECONNREFUSED && // FD_ACCEPT with already 62 clients connected
				u32_Err != WSAESHUTDOWN)      // Sending data to a socket just in the short timespan 
				break;                        //   between shutdown() and closesocket()
		}
	}; // end loop

	CloseSockets();
	if (b_Server) 
		Print("Stop Listening.\r\n");
	else          Print("Connection abandoned.");
	}
	catch(...)
	{
		printf("Error in 'ProcessEvents'");
	}
}

// ##################################################################################################
//                                  PROCESS RECEIVED DATA
// ##################################################################################################

// Mode NORMAL:
// This simple "data processor" prints the data blocks immediately and unchanged as they arrive from the network
void CSocketDlg::ProcessReceivedDataNormal(TCP::cSocket::cMemory* pi_RecvMem)
{
	try
	{
		if (!*&pi_RecvMem) return;
		DWORD u32_Len = pi_RecvMem->GetLength();
		Print("Received: [%lu]", u32_Len);
		char*  s8_Buf = pi_RecvMem->GetBuffer();	
		if (u32_Len <=0) return;
		pi_RecvMem->DeleteLeft(u32_Len);
	}
	catch(...)
	{
		printf("Error in 'ProcessReceivedDataNormal'");
	}
}



int CSocketDlg::ProcessReceivedData(unsigned char* tmp, DWORD size,SOCKET  h_Socket,
									TCP::cSocket::cMemory* pi_RecvMem)
{
	__try
	{
		if (!tmp || size<=0) 
		{
			//   
			if (pi_RecvMem)
			{
				pi_RecvMem->mu32_Len = 0;
				pi_RecvMem->mu32_Size = 0;
				if (pi_RecvMem->ms8_Mem)
				{
					delete pi_RecvMem->ms8_Mem;
					pi_RecvMem->ms8_Mem = NULL;
					pi_RecvMem->ms8_Mem = new char[READ_BUFFER_SIZE];
					pi_RecvMem->mu32_Size = READ_BUFFER_SIZE;
				}
			}
			return -1;
		}
		DWORD readpos = 0;
		unsigned long diff = 0;
		//__try
        {
			
			return 0;
		}
	}
    __except (EXCEPTION_EXECUTE_HANDLER)
        {
            
			LPBYTE lpPage;
            static SYSTEM_INFO si;
            static MEMORY_BASIC_INFORMATION mi;
            static DWORD dwOldProtect;

            //    
            GetSystemInfo(&si);
            
            //   SP
            _asm mov lpPage, esp;

            //    
            VirtualQuery(lpPage, &mi, sizeof(mi));

            //   ,     
            lpPage = (LPBYTE)(mi.BaseAddress)-si.dwPageSize;

            //      
            if (!VirtualFree(mi.AllocationBase,
                            (LPBYTE)lpPage - (LPBYTE)mi.AllocationBase,
                             MEM_DECOMMIT))
				{
                //       
                exit(1);
            }

				//    
				if (!VirtualProtect(lpPage, si.dwPageSize, 
                                PAGE_GUARD | PAGE_READWRITE, 
                                &dwOldProtect))
				{
                exit(1);
            }
				printf("Exception handler %lX", _exception_code()); 
            Sleep(2000);
			return -50;
      }
		

	/*catch(...)
	{
		printf("Error in 'ProcessReceivedData'\n");
		return -3;
	}*/
}


// Mode PREFIX:
// Each datablock comes prefixed with a DWORD which contains the total length of the datablock.
// So it is easy to determine if a block has been received completely.
// The data is accumulated in pi_RecvMem which works like a FIFO memory.
// This is the recommended principle for transmitting binary data.
// To test this mode set SEND_LARGE_DATA to 100 and set READ_BUFFER_SIZE to 30
void CSocketDlg::ProcessReceivedDataPrefix(TCP::cSocket::cMemory* pi_RecvMem)
{
	while (TRUE) // There may arrive multiple datablocks at once -> loop until FIFO is empty
	{
		char*  s8_Buf = pi_RecvMem->GetBuffer();
		DWORD u32_Len = pi_RecvMem->GetLength();
		if (u32_Len < 4)
			return; // There must always be at least 1 Dword

		DWORD u32_Blocksize = ((DWORD*)s8_Buf)[0];
		if (u32_Blocksize > u32_Len)
		{
			Print("%d Bytes in RecvMemory (Blocksize= %d Byte) Waiting for more data...", u32_Len, u32_Blocksize);
			return; // The block is not yet complete -> accumulate more data in pi_RecvMem
		}

		std::string s_String = CopyToString(s8_Buf+4, u32_Blocksize-4);
		Print("Received entire datablock (%d Bytes): '%s'", u32_Blocksize, s_String.c_str());

		// Only delete the data that has been processed and leave the rest.
		// ATTENTION: DeleteLeft(u32_Len) would result in data loss!!
		pi_RecvMem->DeleteLeft(u32_Blocksize);
	}
}

// Mode TELNET:
// This function demonstrates how single characters received from a Telnet client
// are accumulated in pi_RecvMem, which works like a FIFO memory, until a line feed is found.
// When a line is complete it is printed to the screen and deleted from pi_RecvMem.
void CSocketDlg::ProcessReceivedDataTelnet(TCP::cSocket::cMemory* pi_RecvMem)
{
	
}

// Close all open sockets
void CSocketDlg::OnBtnClose() 
{
	if (!mi_Socket.GetSocketCount())
		Print("No Socket open!");
	else
		CloseSockets();
}

// Close all open sockets (if any)
void CSocketDlg::CloseSockets() 
{
	if (mi_Socket.GetSocketCount())
	{
		mi_Socket.Close();
		Print("Socket(s) closed.");
		allpacketscount = 0;
	}
}

// Send a text string to one or multiple destinations
void CSocketDlg::OnBtnSend() 
{
	
}

// Sends data to the given socket
// A "\r\n" in the input string is replaced with a linebreak
// returns FALSE when the sockets have been closed due to a severe error
BOOL CSocketDlg::SendTo(SOCKET h_Socket, std::string s_SendData) 
{
	return true;
}

// Clear the content of the Output Editbox
void CSocketDlg::OnBtnClear() 
{
	
}

// Allows to update the GUI from within an endless loop without needing an extra thread
void CSocketDlg::PumpMessages()
{

}

// When the main dialog is closed: set the mb_DlgClosed flag to abort the ProcessEvents() Thread!
void CSocketDlg::OnClose()
{
	mb_DlgClosed = TRUE;
	mi_Socket.Close();
}

// Appends formatted text to the string ms_Output which is later written to the Output Editbox in the GUI thread
int CSocketDlg::Print(const char *fmt, ...)
{
	try
	{
	//if(TryEnterCriticalSection (&mk_Critical))
	EnterCriticalSection(&mk_Critical);
	{
		//do something that must be synh
		va_list vl;
		int size;
		SYSTEMTIME st;
		GetLocalTime(&st);
		char buff[64] = "\0";
		memset(&buff,0,sizeof(buff));
		sprintf(buff, 
			"%0.2d-%0.2d-%0.4d %0.2d:%0.2d:%0.2d.%0.3d", 
			st.wDay,
			st.wMonth,
			st.wYear,
			st.wHour, 
			st.wMinute, 
			st.wSecond,
			st.wMilliseconds);

		enum { INITIAL_BUFFER_SIZE = 512 };
		{
			char buffer[INITIAL_BUFFER_SIZE];
			va_start(vl, fmt);
			size = vsnprintf(buffer, INITIAL_BUFFER_SIZE, fmt, vl);
			va_end(vl);
			if (size < INITIAL_BUFFER_SIZE)
			{
			   // ms_Output += (buffer);
				if (strlen(buffer)>0)
				printf("%s %s\n",buff,buffer);
				//printf("%s",buffer);
				LeaveCriticalSection(&mk_Critical);
				return size;
			}
		}
		size += sizeof('\0');
		char* tmp = (char*)malloc(sizeof(char)*size);
		if (!tmp)
		{
			LeaveCriticalSection(&mk_Critical);
			return 0;
		}
		va_start(vl, fmt);
		size = vsnprintf(tmp, size, fmt, vl);
		va_end(vl);
		//ms_Output += (tmp);
		if (strlen(tmp)>0)
		printf("%s %s\n",buff,tmp);
		//printf("%s",tmp);
		free(tmp);
		LeaveCriticalSection(&mk_Critical);
		

		return size;		
		
	}
	
	return 0;
	}
	catch(...)
	{
		printf("Error in 'Print'");
	}
}
	
// Copies the not zero terminated data in s8_Buf into a std::string
// u32_Bytes always specifies the length in bytes no matter if compiled as Unicode or MBCS
std::string CSocketDlg::CopyToString(char* s8_Buf, DWORD u32_Bytes)
{
	return "";
}

// Format the display string for the given socket
// returns "Server (192.168.1.100)" or "Client 71C (192.168.1.100)"
std::string CSocketDlg::FormatDisplayName(SOCKET h_Socket)
{
	std::string s_IP = FormatIP(mi_SocketList.GetValueByKey(h_Socket));
	std::string s_Disp;
	char tmp[128]="\0";
	if (mi_Socket.GetState() & TCP::cSocket::E_Server) 
	sprintf(tmp,"Client %X (%s)",h_Socket, s_IP.c_str());
	else
	sprintf(tmp,"Server (%s)", s_IP.c_str());
	
	s_Disp = tmp;
	return s_Disp.c_str();
}

// Formats an IP address "192.168.1.100"
std::string CSocketDlg::FormatIP(DWORD u32_IP)
{
	BYTE* pu8_Addr = (BYTE*)&u32_IP;

	std::string s_IP;
	char tmp[128]="\0";
	sprintf(tmp,"%d.%d.%d.%d", pu8_Addr[0],pu8_Addr[1],pu8_Addr[2],pu8_Addr[3]);
	s_IP = tmp;
	return s_IP.c_str();
}

// Get a human readable error message for an API error code
std::string CSocketDlg::GetErrMsg(DWORD u32_Error)
{
	// Some translations of error codes are really stupid --> show the original error code.
	std::string s_Code;
	switch (u32_Error)
	{
		case WSAEINTR:                s_Code = "WSAEINTR"; break;
		case WSAEBADF:                s_Code = "WSAEBADF"; break;
		case WSAEACCES:               s_Code = "WSAEACCES"; break;
		case WSAEFAULT:               s_Code = "WSAEFAULT"; break;
		case WSAEINVAL:               s_Code = "WSAEINVAL"; break;
		case WSAEMFILE:               s_Code = "WSAEMFILE"; break;
		case WSAEWOULDBLOCK:          s_Code = "WSAEWOULDBLOCK"; break;
		case WSAEINPROGRESS:          s_Code = "WSAEINPROGRESS"; break;
		case WSAEALREADY:             s_Code = "WSAEALREADY"; break;
		case WSAENOTSOCK:             s_Code = "WSAENOTSOCK"; break;
		case WSAEDESTADDRREQ:         s_Code = "WSAEDESTADDRREQ"; break;
		case WSAEMSGSIZE:             s_Code = "WSAEMSGSIZE"; break;
		case WSAEPROTOTYPE:           s_Code = "WSAEPROTOTYPE"; break;
		case WSAENOPROTOOPT:          s_Code = "WSAENOPROTOOPT"; break;
		case WSAEPROTONOSUPPORT:      s_Code = "WSAEPROTONOSUPPORT"; break;
		case WSAESOCKTNOSUPPORT:      s_Code = "WSAESOCKTNOSUPPORT"; break;
		case WSAEOPNOTSUPP:           s_Code = "WSAEOPNOTSUPP"; break;
		case WSAEPFNOSUPPORT:         s_Code = "WSAEPFNOSUPPORT"; break;
		case WSAEAFNOSUPPORT:         s_Code = "WSAEAFNOSUPPORT"; break;
		case WSAEADDRINUSE:           s_Code = "WSAEADDRINUSE"; break;
		case WSAEADDRNOTAVAIL:        s_Code = "WSAEADDRNOTAVAIL"; break;
		case WSAENETDOWN:             s_Code = "WSAENETDOWN"; break;
		case WSAENETUNREACH:          s_Code = "WSAENETUNREACH"; break;
		case WSAENETRESET:            s_Code = "WSAENETRESET"; break;
		case WSAECONNABORTED:         s_Code = "WSAECONNABORTED"; break;
		case WSAECONNRESET:           s_Code = "WSAECONNRESET"; break;
		case WSAENOBUFS:              s_Code = "WSAENOBUFS"; break;
		case WSAEISCONN:              s_Code = "WSAEISCONN"; break;
		case WSAENOTCONN:             s_Code = "WSAENOTCONN"; break;
		case WSAESHUTDOWN:            s_Code = "WSAESHUTDOWN"; break;
		case WSAETOOMANYREFS:         s_Code = "WSAETOOMANYREFS"; break;
		case WSAETIMEDOUT:            s_Code = "WSAETIMEDOUT"; break;
		case WSAECONNREFUSED:         s_Code = "WSAECONNREFUSED"; break;
		case WSAELOOP:                s_Code = "WSAELOOP"; break;
		case WSAENAMETOOLONG:         s_Code = "WSAENAMETOOLONG"; break;
		case WSAEHOSTDOWN:            s_Code = "WSAEHOSTDOWN"; break;
		case WSAEHOSTUNREACH:         s_Code = "WSAEHOSTUNREACH"; break;
		case WSAENOTEMPTY:            s_Code = "WSAENOTEMPTY"; break;
		case WSAEPROCLIM:             s_Code = "WSAEPROCLIM"; break;
		case WSAEUSERS:               s_Code = "WSAEUSERS"; break;
		case WSAEDQUOT:               s_Code = "WSAEDQUOT"; break;
		case WSAESTALE:               s_Code = "WSAESTALE"; break;
		case WSAEREMOTE:              s_Code = "WSAEREMOTE"; break;
		case WSASYSNOTREADY:          s_Code = "WSASYSNOTREADY"; break;
		case WSAVERNOTSUPPORTED:      s_Code = "WSAVERNOTSUPPORTED"; break;
		case WSANOTINITIALISED:       s_Code = "WSANOTINITIALISED"; break;
		case WSAEDISCON:              s_Code = "WSAEDISCON"; break;
		case WSAENOMORE:              s_Code = "WSAENOMORE"; break;
		case WSAECANCELLED:           s_Code = "WSAECANCELLED"; break;
		case WSAEINVALIDPROCTABLE:    s_Code = "WSAEINVALIDPROCTABLE"; break;
		case WSAEINVALIDPROVIDER:     s_Code = "WSAEINVALIDPROVIDER"; break;
		case WSAEPROVIDERFAILEDINIT:  s_Code = "WSAEPROVIDERFAILEDINIT"; break;
		case WSASYSCALLFAILURE:       s_Code = "WSASYSCALLFAILURE"; break;
		case WSASERVICE_NOT_FOUND:    s_Code = "WSASERVICE_NOT_FOUND"; break;
		case WSATYPE_NOT_FOUND:       s_Code = "WSATYPE_NOT_FOUND"; break;
		case WSA_E_NO_MORE:           s_Code = "WSA_E_NO_MORE"; break;
		case WSA_E_CANCELLED:         s_Code = "WSA_E_CANCELLED"; break;
		case WSAEREFUSED:             s_Code = "WSAEREFUSED"; break;
		case WSAHOST_NOT_FOUND:       s_Code = "WSAHOST_NOT_FOUND"; break;
		case WSATRY_AGAIN:            s_Code = "WSATRY_AGAIN"; break;
		case WSANO_RECOVERY:          s_Code = "WSANO_RECOVERY"; break;
		case WSANO_DATA:              s_Code = "WSANO_DATA"; break;
		case WSA_IO_PENDING:          s_Code = "WSA_IO_PENDING"; break;
		case WSA_IO_INCOMPLETE:       s_Code = "WSA_IO_INCOMPLETE"; break;
		case WSA_INVALID_HANDLE:      s_Code = "WSA_INVALID_HANDLE"; break;
		case WSA_INVALID_PARAMETER:   s_Code = "WSA_INVALID_PARAMETER"; break;
		case WSA_NOT_ENOUGH_MEMORY:   s_Code = "WSA_NOT_ENOUGH_MEMORY"; break;
		case WSA_OPERATION_ABORTED:   s_Code = "WSA_OPERATION_ABORTED"; break;
		
		default:
			char tmp[128] = "\0";
			sprintf(tmp,"Code %u", u32_Error); 
			s_Code= tmp;
			break;
	}

	std::string s_Out;
	const DWORD BUFLEN = 1000;
	TCHAR t_Buf[BUFLEN];
	char temp[1024] = "\0";
	memset(&temp,0,sizeof(temp));
	if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, u32_Error, 0, t_Buf, BUFLEN, 0))
		sprintf(temp,"%s: %s", s_Code.c_str(), t_Buf);
	else 
		sprintf(temp,"%s: Windows has no explanation for this erron!", s_Code.c_str());

	s_Out = temp;
	return s_Out.c_str();
}

std::string PrintLastError (DWORD errCode, const char *msg /* = "Error occurred" */) {
        char *err;
        if (!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                           NULL,
                           errCode,
                           MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // default language
                           (LPTSTR) &err,
                           0,
                           NULL))
            return msg;

        static char buffer[1024];
        _snprintf(buffer, sizeof(buffer), "ERROR: %s: %s", msg, err);
        LocalFree(err);
		return buffer;
}
